-- Random fun things to do while fishing
--
-- Turn on the fish finder
-- Change your title to "Salty"
-- Bring out a "fishing buddy"

local FL = LibStub("LibFishing-1.0");

local SALTY_TITLE = 51;
local PETSETTING = "FishingBuddy";

-- the spell ids for the fishing pets
-- are these as unique as the critter ids? Should be...
local FishingPets = {};
FishingPets[33050] = -1; -- Magical Crawdad
FishingPets[46425] = -1; -- Snarly
FishingPets[46426] = -1; -- Chuck
FishingPets[43697] = -1; -- Toothy
FishingPets[43698] = -1; -- Muckbreath
FishingPets[59250] = -1; -- Giant Sewer Rat
FishingPets[62561] = -1; -- Strand Crawler

-- Debugging
FishingBuddy.FishingPets = FishingPets;

local havepet = 0;

local function FindCurrentPet()
   for sid,id in pairs(FishingPets) do
      FishingPets[sid] = -1;
   end
   havepet = 0;
   local n = GetNumCompanions("CRITTER");
   local pet = 0;
   for id=1,n do
      local _, _, cSpellID, _, here = GetCompanionInfo("CRITTER", id);
      if ( here ) then
         pet = id;
      end
      if ( FishingPets[cSpellID] ) then
         FishingPets[cSpellID] = id;
         havepet = 1;
      end
   end
   return pet;
end

local function PetMenuSelected()
   FindCurrentPet();
   if ( havepet == 1 ) then
      local show = 1;
      local first = nil;
      local pet = FishingBuddy.GetSetting(PETSETTING);
      local petid;
      for sid,id in pairs(FishingPets) do
         if ( id > 0 ) then
            if ( not first ) then
               first = sid;
               petid = id;
            end
            if ( sid == pet ) then
               first = sid;
               petid = id;
               break;
            end
            show = show + 1;
         end
      end
      local cName;
      if ( pet == 0 ) then
         FishingBuddy.SetSetting(PETSETTING, 0);
         -- show should already be the last item in the menu
         cName = NONE;
      else
         _, cName, _, _, _ = GetCompanionInfo("CRITTER", petid);
      end
      UIDropDownMenu_SetText(FishingFluffPetMenu, cName);
      UIDropDownMenu_SetSelectedValue(FishingFluffPetMenu, show);
   end
end

local function PetMenuSetup()
   FindCurrentPet();
   local picked = FishingBuddy.GetSetting(PETSETTING);
   local info;
   for sid,id in pairs(FishingPets) do
      if ( id > 0 ) then
         local cID, cName, cSpellID, icon, here = GetCompanionInfo("CRITTER", id);
         info = {};
         info.text = cName;
         info.func = function() FishingBuddy.SetSetting(PETSETTING, cSpellID); PetMenuSelected(); end;
         info.checked = ( picked == cSpellID );
         UIDropDownMenu_AddButton(info);
      end
   end
   info = {};
   info.text = NONE;
   info.func = function() FishingBuddy.SetSetting(PETSETTING, 0); PetMenuSelected(); end;
   info.checked = ( picked == 0 );
   UIDropDownMenu_AddButton(info);
end

local FluffEvents = {};

local resetID = nil;
local resetPVP = nil;
local resetTitle = nil;
local resetPet = nil;

FluffEvents[FBConstants.FISHING_ENABLED_EVT] = function()
   if ( FishingBuddy.GetSettingBool("FishingFluff")) then
      if ( FishingBuddy.GetSettingBool("FindFish") ) then
         local findid = FL:GetFindFishID();
         if ( findid ) then
            resetID = FL:GetTrackingID();
            if ( resetID == findid ) then
               resetID = nil;
            else
               SetTracking(findid);
            end
         end
      end
      if ( FishingBuddy.GetSettingBool("TurnOffPVP") ) then
         if (1 == GetPVPDesired() ) then
            resetPVP = true;
            SetPVP(0);
         end
      end
      if ( FishingBuddy.GetSettingBool("FishingTitle") and IsTitleKnown(SALTY_TITLE) ) then
         resetTitle = GetCurrentTitle();
         SetCurrentTitle(SALTY_TITLE);
      end
      if ( not (IsFlying() or IsMounted()) ) then
         local pet = FishingBuddy.GetSetting(PETSETTING);
         if ( pet > 0 ) then
            local nowpet = FindCurrentPet();
            if ( pet ~= nowpet ) then
               resetPet = nowpet;
               CallCompanion("CRITTER", FishingPets[pet]);
            end
         end
      end
   end
end

FluffEvents[FBConstants.FISHING_DISABLED_EVT] = function(started)
   if ( resetID ) then
      SetTracking(resetID);
      resetID = nil;
   end
   if ( resetPVP ) then
      SetPVP(1);
      resetPVP = nil;
   end
   if ( resetTitle ) then
      SetCurrentTitle(resetTitle);
      resetTitle = nil;
   end
   if ( resetPet ) then
      if ( not (IsFlying() or IsMounted()) ) then
         if ( resetPet > 0 ) then
            CallCompanion("CRITTER", resetPet);
         else
            DismissCompanion("CRITTER");
         end
      end
      resetPet = nil;
   end
end

local FluffOptions = {
   ["FishingFluff"] = {
      ["text"] = FBConstants.CONFIG_FISHINGFLUFF_ONOFF,
      ["tooltip"] = FBConstants.CONFIG_FISHINGFLUFF_INFO,
      ["v"] = 1,
      ["default"] = 1 },
   ["FindFish"] = {
      ["text"] = FBConstants.CONFIG_FINDFISH_ONOFF,
      ["tooltip"] = FBConstants.CONFIG_FINDFISH_INFO,
      ["v"] = 1,
      ["deps"] = { ["FishingFluff"] = "d" },
      ["default"] = 1 },
   ["FishingBuddy"] = {
      ["tooltip"] = FBConstants.CONFIG_FISHINGBUDDY_INFO,
      ["setup"] = PetMenuSelected,
      ["visible"] =
         function()
            return havepet;
         end,
      ["button"] = "FishingFluffPetMenuHolder",
      ["deps"] = { ["FishingFluff"] = "h" },
      ["default"] = 0 },
   ["FishingTitle"] = {
      ["text"] = FBConstants.CONFIG_FISHINGTITLE_ONOFF,
      ["tooltip"] = FBConstants.CONFIG_FISHINGTITLE_INFO,
      ["v"] = 1,
      ["deps"] = { ["FishingFluff"] = "d" },
      ["default"] = 1 },
   ["DrinkHeavily"] = {
      ["text"] = FBConstants.CONFIG_DRINKHEAVILY_ONOFF,
      ["tooltip"] = FBConstants.CONFIG_DRINKHEAVILY_INFO,
      ["v"] = 1,
      ["m"] = 1,
      ["deps"] = { ["FishingFluff"] = "d" },
      ["default"] = 1 },
};

FluffEvents["VARIABLES_LOADED"] = function(started)
   FishingBuddy.OptionsFrame.HandleOptions(GENERAL, nil, FluffOptions);
   local f = CreateFrame("FRAME", "FishingFluffPetMenuHolder");
   local menu = CreateFrame("FRAME", "FishingFluffPetMenu", f, "FishingBuddyDropDownMenuTemplate");
   menu:SetPoint("TOPRIGHT", f, "TOPRIGHT", 2, 2);
   UIDropDownMenu_Initialize(menu, PetMenuSetup);
   local label = getglobal("FishingFluffPetMenuLabel");
   label:SetText(PET_TYPE_PET..": ");
   UIDropDownMenu_SetWidth(menu, 120);
   f:SetWidth(menu:GetWidth() + label:GetWidth() - 8);
   f:SetHeight(menu:GetHeight() + label:GetHeight());
   PetMenuSelected();
end


FishingBuddy.API.RegisterHandlers(FluffEvents);
